// SLB - Super Light Blog
//
// Copyright (c) 2005 Satoshi Fukutomi <info@fuktommy.com>.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

//
// Send request
//
function XMLrequest() {
    var xmlhttp;
    try {
        xmlhttp = new XMLHttpRequest();
    } catch (e) {
        xmlhttp = new ActiveXObject("Microsoft.XMLHTTP");
    }
    if (xmlhttp) {
        return xmlhttp;
    } else {
        return null;
    }
}
// End of XMLrequest


//
// Ajax RSS Read
//
function RSSread() {
}

RSSread.prototype.read = function (uri) {
    this.xmlrequest = new XMLrequest();
    var self = this;
    this.xmlrequest.onreadystatechange = function () {
        if (! (self.xmlrequest.readyState == 4 &&
               self.xmlrequest.status == 200)) {
            return;
        }
        var xml = self.xmlrequest.responseXML;
        self.parseXML(xml);
    }
    this.xmlrequest.open("GET", uri, true);
    this.xmlrequest.send(null);
}

// End of RSSread


//
// Inherit
// from http://www.tokumaru.org/JavaScript/auto_inheritance_2.htm
//
function inherit(subClass, superClass) {
    for (var prop in superClass.prototype) {
        subClass.prototype[prop] = superClass.prototype[prop];
    }
}
// End of Inherit


//
// Get Element Value
//
function getValue(object, tagname) {
    var element = object.getElementsByTagName(tagname)[0];
    if (element) {
        return element.firstChild.nodeValue;
    } else {
        return null;
    }
}
// End of getValue


//
// Entry (An article)
//
function Entry(item) {
    this.title = getValue(item, "title");
    this.link = getValue(item, "link");
    this.date = getValue(item, "dc:date") || getValue(item, "date");
    this.description = getValue(item, "description");
    this.content = getValue(item, "content:encoded") ||
                   getValue(item, "encoded");
    this.id = this.link.substring(this.link.search("#")+2)
}

function DummyEntry() {
    this.title = "";
    this.link = "";
    this.date = "";
    this.description = "";
    this.content = "";
    this.id = "";
}
// End of Entry


//
// Entry Cache
//
function EntryCache() {
    this.cache = {};
    this.index = [];
}

EntryCache.prototype.append = function (entry) {
    if (this.cache[entry.id]) {
        if (! entry.content) {
            return;
        } else {
            var tmp = [];
            for (var i=0; i<this.index.length; i++) {
                if (this.index[i] != entry.id) {
                    tmp.push(this.index[i]);
                }
            }
            this.index = tmp;
        }
    }
    this.index.push(entry.id);
    this.cache[entry.id] = entry;
    if (this.index.length > cacheSize) {
        var id = this.index.shift();
        this.cache.remove(id);
    }
}

// End of EntryCache


//
// Recent RSS
//
function RecentRSS(topPageMode) {
    this.topPageMode = topPageMode;
}

RecentRSS.prototype.parseXML = function (xml) {
    var recentList = new RecentList(xml);
    if (this.topPageMode) {
        var recentEntry = new EntryList(xml, recentSize);
    }
}

inherit(RecentRSS, RSSread);
// End of RecentRSS


//
// Archives RSS
//
function ArchivesRSS() {
}

ArchivesRSS.prototype.parseXML = function (xml) {
    var archiveList = new ArchivesList(xml);
}

inherit(ArchivesRSS, RSSread);
// End of ArchivesRSS


//
// Monthly RSS
//
function MonthlyRSS() {
}

MonthlyRSS.prototype.parseXML = function (xml) {
    var entryList = new EntryList(xml);
}

inherit(MonthlyRSS, RSSread);
// End of MothlyRSS


//
// Entry RSS
//
function EntryRSS() {
}

EntryRSS.prototype.parseXML = function (xml) {
    var entry = new Entry(xml.getElementsByTagName("item")[0]);
    entryCache.append(entry);
    var entryView = new EntryView(entry);
}

inherit(EntryRSS, RSSread);
// End of EntryRSS


