// SLB - Super Light Blog
//
// Copyright (c) 2005 Satoshi Fukutomi <info@fuktommy.com>.
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

// Config
var blogTitle = "blog.fuktommy.com";
var cgiURI = "blog.cgi";
var rssURI = "xml/rss";
var archivesURI = "xml/archives.rdf";
var entryURI = "xml";
var recentSize = 10;
var cacheSize = 500;
var timezoneOffset = -540;

var entryCache;

function localDate(date) {
    return date.replace("T", " ").replace(/[-+]..:../, "");
}


//
// Entry
//
Entry.prototype.toString = function () {
    var buf = '<div class="entry">' +
              '<h2><a href="' + this.link + '" title="Permalink"' +
              ' onclick="viewEntry(\'' + this.id + '\'); return false;">' +
              this.title + '</a></h2>';
    if (this.content) {
        buf += this.content;
    } else if (this.description) {
        buf += '<p class="description">' + this.description + "</p>";
    }
    buf += '<ul class="feedback">' +
           '<li><a href="http://b.hatena.ne.jp/entry/' +
           this.link.replace("#", "%23") + '">Comments</a></li>' +
           '<li><a href="http://blogsearch.google.com/blogsearch?q=' +
           'link%3A' + encodeURIComponent(this.link) +'">' +
           'Links to this Entry</a></li>' +
           '<li>' + localDate(this.date) + '</li>';
    if (entryCache.cache[this.id] && entryCache.cache[this.id].content) {
        buf += '<li><a href=""' +
               ' onclick="viewEdit(\'' + this.id + '\'); return false;">' +
               'Edit</a></li>';
        }
    buf += '</ul></div>';
    return buf;
}

Entry.prototype.toAnchor = function () {
    return '<a href="' + this.link + '"' +
           ' onmouseover="peepEntry(\'' + this.id + '\');"' +
           ' onclick="hidePeep(); viewEntry(\'' + this.id + '\');' +
           ' return false;">' +
           this.title + '</a>';
}
// End of Entry


//
// Recent Posts List
//
function RecentList(xml) {
    var items = xml.getElementsByTagName("item");
    this.panel = document.getElementById("recent");
    var buf = "<ul>";
    for (i=0; (i<items.length)&&(i<recentSize); i++) {
        var entry = new Entry(items[i]);
        entryCache.append(entry);
        buf += '<li><a href="' + entry.link + '"' +
               ' onclick="viewEntry(\'' + entry.id + '\'); return false;">' +
               entry.title + '</a></li>';
    }
    buf += "</ul>";
    this.panel.innerHTML = buf;
}
// End of RecentList

//
// Entry List
//
function EntryList(xml, size) {
    var items = xml.getElementsByTagName("item");
    this.panel = document.getElementById("main");
    var buf = "";
    for (i=0; i<items.length; i++) {
        if (size && (i>=size)) {
            break;
        }
        var entry = new Entry(items[i]);
        entryCache.append(entry);
        buf +=  entry.toString();
    }
    document.title = blogTitle;
    this.panel.innerHTML = buf;
    window.scroll(0, 0);
}
// End of EntryList


//
// Archives List
//
function ArchivesList(xml) {
    var items = xml.getElementsByTagName("item");
    this.panel = document.getElementById("archive");
    var buf = "<ul>" +
              '<li><a href="" onclick="viewRecent(); return false;">' +
              'Current Entries</a></li>';
    for (i=0; i<items.length; i++) {
        var entry = new Entry(items[i]);
        buf += '<li><a href="' + entry.link +'"' +
               ' onclick="viewArchives(\'' + entry.id + '\');return false;">' +
               entry.title + '</a></li>';
    }
    buf += "</ul>";
    this.panel.innerHTML = buf;
}
// End of ArchivesList


//
// View Mothly Archive
//
function viewArchives(id) {
    var monthlyRSS = new MonthlyRSS();
    monthlyRSS.read(entryURI + "/" + id + "/index.rdf");
}
// End of viewArchives


//
// Entry View
//
function EntryView(entry) {
    this.panel = document.getElementById("main");
    document.title = entry.title;
    this.panel.innerHTML = entry.toString();
}
// Enf of EntryView


//
// View Recent Posts List
//
function viewRecent() {
    var recentRSS = new RecentRSS(true);
    recentRSS.read(rssURI);    
}
// End of viewRecent


//
// View an Entry
//
function viewEntry(id) {
    if (entryCache.cache[id] && entryCache.cache[id].content) {
        var entry = entryCache.cache[id];
        var entryView = EntryView(entry);
    } else {
        var date = new Date();
        date.setTime(id * 1000 - timezoneOffset);
        var year = date.getYear();
        if (year < 1900) {
            year += 1900;
        }
        var month = date.getMonth() + 1;
        if (month < 10) {
            month = "0" + month;
        }
        var uri = entryURI + "/" + year + "-" + month + "/" + id + ".rdf";
        var entryRSS = new EntryRSS();
        entryRSS.read(uri);
    }
    window.scroll(0, 0);
}
// End of viewEntry


//
// Peep an Entry
//
function peepEntry(id) {
    if (entryCache.cache[id]) {
        var panel = document.getElementById("peep");
        if (panel) {
            panel.innerHTML = entryCache.cache[id].toString();
        }
    }
}

function hidePeep() {
    var panel = document.getElementById("peep");
    if (panel) {
        panel.innerHTML = "";
    }
}

// End of popupEntry


//
// Edit Form
//
function EditForm(entry) {
    this.entry = entry;
    this.panel = document.getElementById("main");
    var buf = "";
    var title = entry.title.replace(/"/g, '&quote;');
    var content = entry.content.replace(/&/g, "&amp;");
    content = content.replace(/</g, "&lt;");
    content = content.replace(/>/g, "&gt;");
    buf = '<form id="editform"><p>' +
          '<input type="hidden" name="id" value="' + entry.id + '" />' +
          'Edit: <a href="' + entry.link + '"' +
          ' onclick="viewEntry(\'' + entry.id + '\'); return false">' +
          entry.title + '</a><br />' +
          'Title: ' +
          '<input name="title" value="'+ title + '" size="60" />' +
          '<br />' +
          'Password: ' +
          '<input type="password" name="passwd" value="" />' +
          '<button type="button" id="savebutton">Save</button>' +
          '<button type="button" id="previewbutton">Preview</button>' +
          '<br />' +
          '<textarea name="body" cols="60" rows="20">' + 
          content + '</textarea>' +
          '</p></form>' +
          '<div id="preview"></div>';
    document.title = this.entry.title;
    this.panel.innerHTML = buf;
    var savebutton = document.getElementById("savebutton");
    savebutton.onclick = saveEntry;
    var previewbutton = document.getElementById("previewbutton");
    previewbutton.onclick = updatePreview;
    updatePreview();
}
// End of EditForm

//
// Update Preview
//
function updatePreview() {
    var preview = document.getElementById("preview");
    var entrytitle = document.getElementById("editform").title;
    var entrybody = document.getElementById("editform").body;
    var content = entrybody.value.replace(/&lt;/g, "<");
    content = content.replace(/&gt;/g, ">");
    content = content.replace(/&amp;/g, "&");
    preview.innerHTML = '<h2>' + entrytitle.value + '</h2>' + content;
}

//
// View Edit Mode
//
function viewEdit(id) {
    if (id && entryCache.cache[id] && entryCache.cache[id].content) {
        var editForm = new EditForm(entryCache.cache[id]);
    } else {
        var editForm = new EditForm(new DummyEntry());
    }
    window.scroll(0, 0);
}
// End of viewEdit


//
// Save Entry
//
function saveEntry() {
    xmlrequest = XMLrequest();
    if (! xmlrequest) {
        return;
    }
    var buf = "";
    var form = document.getElementById("editform");
    buf += "id=" + form.id.value;
    buf += "&title=" + encodeURIComponent(form.title.value);
    buf += "&passwd=" + encodeURIComponent(form.passwd.value);
    buf += "&body=" + encodeURIComponent(form.body.value);
    xmlrequest.onreadystatechange = function () {
        try {
            if (xmlrequest.readyState != 4) {
                return;
            } else if (xmlrequest.status != 200) {
                alert("HTTP Error (status=" + xmlrequest.status + ").");
                return;
            }
        } catch (e) {
                alert("HTTP Error (exception).");
                return;
        }
        var xml = xmlrequest.responseXML;
        if (! xml) {
            alert("XML Error.");
        } else if (xml.documentElement.nodeName == "error") {
            var err = "";
            try {
                err = xml.documentElement.firstChild.nodeValue;
            } catch (e) {
                err = "Error";
            }
            alert(err);
        } else {
            var recentRSS = new RecentRSS(true);
            recentRSS.read(rssURI);
        }
    }
    xmlrequest.open("POST", cgiURI, true);
    try {
        xmlrequest.setRequestHeader("Content-Type",
                "application/x-www-form-urlencoded; charset=utf-8")
    } catch (e) {
    }
    xmlrequest.send(buf);
}
// Enf of saveEntry


//
// Search Cache
//
function searchCache() {
    var q = document.getElementById("searchform").q.value;
    var query = split(q.toLowerCase());
    hidePeep();
    if (query.length == 0) {
        viewRecent();
        return;
    }
    var panel = document.getElementById("main");
    var ids = entryCache.index.sort().reverse();
    var buf = '<ul class="result">';
    for (var i=0; i<ids.length; i++) {
        var e = entryCache.cache[ids[i]];
        var target = e.title + " " + (e.content || e.description);
        if (andSearch(query, target)) {
            buf += '<li>' + e.toAnchor() +
                   '  (' + localDate(e.date) + ')</li>';
        }
    }
    buf += '</ul>' +
           '<div id="peep"></div>';
    panel.innerHTML = buf;
}
// End of searchCache


//
// Main
//
function main() {
    entryCache = new EntryCache();
    if (location.hash) {
        var recentRSS = new RecentRSS(false);
        if (location.hash.search("#a") == 0) {
            viewArchives(location.hash.substring(2));
        } else if (location.hash.search("#e") == 0) {
            viewEntry(location.hash.substring(2));
        }
        location.hash = "#";
    } else {
        var recentRSS = new RecentRSS(true);
    }
    recentRSS.read(rssURI);
    var archivesRSS = new ArchivesRSS();
    archivesRSS.read(archivesURI);
}
